import json
import requests
from bs4 import BeautifulSoup
from .utils import editionMap, topicMap, langMap
from .userexception import NotFound


class gnfetcher:

    def __init__(self, edition='United States (English)',
                 topic='top stories', location=None,
                 query=None, language='english'):
        '''
        constructor function
        '''
        # list of editions and topics
        self.editions = list(editionMap)
        self.topics = list(topicMap)
        self.languages = list(langMap)

        # default parameter values
        self.edition = edition
        self.topic = topic
        self.location = location
        self.query = query
        self.language = language

        # parameters to be passed in HTTP request
        self.params = {'output': 'atom',
                       'ned': self.edition,
                       'topic': self.topic,
                       'geo': self.location,
                       'q': self.query,
                       'hl': self.language}

    def get_config(self):
        '''
        function to get current configuration
        '''
        config = {
            'edition': self.edition,
            'topic': self.topic,
            'language': self.language,
            'location': self.location,
            'query': self.query
        }
        return config

    def reset(self):
        '''
        function to reset the parameters
        '''
        self.edition = 'United States (English)'
        self.language = 'english'
        self.location = None
        self.query = None
        self.topic = 'top stories'

    def get_news(self):
        '''
        function to fetch news articles
        '''
        status = self.set_params()
        # params not set properly
        if status is False:
            return

        soup = self.load_feed()
        articles = self.scrape_feed(soup)
        return articles

    def set_params(self):
        '''
        function to set params for HTTP request
        '''

        # setting edition
        try:
            self.params['ned'] = editionMap[self.edition]
        except KeyError:
            print("{self.edition} edition not found.\nUse editions attribute to get list of editions.")
            return False

        # setting topic
        try:
            self.params['topic'] = topicMap[self.topic]
        except KeyError:
            print(f"{self.topic} topic not found.\n"
                  f"Use topics attribute to get list of topics.")
            return False

        # setting language
        try:
            self.params['hl'] = langMap[self.language]
        except KeyError:
            print(f"{self.language} language not found.\n"
                  f"Use langugaes attribute to get list of languages.")
            return False

        # setting query
        if self.query is not None:
            self.params['q'] = self.query
            # topic overrides query parameter. So, clearing it.
            self.params['topic'] = None

        # setting location
        if self.location is not None:
            self.params['geo'] = self.location
            # topic overrides location parameter. So, overriding it.
            self.params['topic'] = None

        # params setting successful
        return True

    def load_feed(self):
        '''
        function to load atom feed
        '''
        url = "https://news.google.com/news"
        resp = requests.get(url, params=self.params)
        soup = BeautifulSoup(resp.content, 'html5lib')
        return soup

    def scrape_feed(self, soup):
        '''
        function to scrape atom feed
        '''
        entries = soup.findAll('entry')
        articles = []

        for entry in entries:
            article = {}
            article['title'] = entry.title.text
            article['link'] = entry.link['href'].split('&url=')[1]
            article['releasedAt'] = entry.updated.text

            try:
                string = entry.content.text.split('src=\"')[1].split('\"')[0]
                article['img'] = "https:" + string
            except Exception:
                article['img'] = None
                pass
            articles.append(article)
        try:
            if len(articles) == 0:
                raise NotFound
        except NotFound:
                print("The articles for the given response are not found.")
                return
        return articles



import click
from gnfetcher import gnfetcher

client = gnfetcher()

@click.command()
@click.option("--config",is_flag=True,help="shows default config")

@click.option("--query",default=None,help="shows news about query given")
@click.option("--edition",default="United States (English)",help="shows news of edition given, default=United States (English)")
@click.option("--topic",default="top stories",help="shows topic given, default=top stories")
@click.option("--location",default=None,help="shows news from location given")
@click.option("--language",default="english",help="shows news in language given, default is english")

@click.option("--sheditions",is_flag=True,help="shows list of available editions")
@click.option("--shtopics",is_flag=True,help="shows list of available topics")
@click.option("--shlangs",is_flag=True,help="shows list of available languages")

def cli(config,query,edition,topic,location,language,shlangs,shtopics,sheditions):
	""" CLI to get news """

	client.query = query
	client.edition = edition
	client.topic = topic
	client.location = location
	client.language = language

	if config:
		conf = client.get_config()
		click.echo("The default configuration : ")
		for keys,value in conf.items():
			click.echo(str(keys)+" : "+str(value))

	elif shlangs:
		langs = client.languages
		click.echo("The languages supported : ")
		for l in langs:
			click.echo(l)

	elif sheditions:
		editions = client.editions
		click.echo("The editions available : ")
		for e in editions:
			click.echo(e)

	elif shtopics:
		tps = client.topics
		click.echo("The topics available : ")
		for t in tps:
			click.echo(t)
	else:
		neews = client.get_news()
		for n in neews:
			content = "{}\n{}".format(n['title'],n['link'])
			click.echo(content)
			click.echo("\n")



# -*- coding: utf-8 -*-
from setuptools import setup

def readme():
	try:
	    with open('README.rst') as f:
	        return f.read()
	except:
		pass

setup(name = 'gnfetcher',
      version = '1.0.1',
      classifiers = [
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
	    'Programming Language :: Python :: 2',
	    'Programming Language :: Python :: 2.6',
	    'Programming Language :: Python :: 2.7',
	    'Programming Language :: Python :: 3',
	    'Programming Language :: Python :: 3.3',
	    'Programming Language :: Python :: 3.4',
	    'Programming Language :: Python :: 3.5',
        'Topic :: Internet',
      ],
      keywords = 'google news feed python tool',
      scripts = ['bin/gnews-gui'], 
      description = 'Python client for Google News feed.',
      long_description = readme(),
      url = 'http://github.com/aadu999/gnfetcher',
      author = 'Adarsh Raveendra',
      author_email = 'hello@adarsh.online',
      license = 'MIT',
      packages = ['gnfetcher', 'gnfetcher.scripts'],
      install_requires = ['requests', 'bs4', 'html5lib', 'Click'], 
      include_package_data = True,
      zip_safe = False,
      entry_points='''
        [console_scripts]
        gnews=gnfetcher.scripts.gnews:cli
      ''',
      )
